//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Core/Element/SpecularElement.cpp
//! @brief     Implements the class SpecularElement.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Core/Element/SpecularElement.h"
#include "Device/Beam/Beam.h"
#include "Device/Detector/IDetector.h"
#include "Device/Instrument/Instrument.h"
#include "Resample/Slice/KzComputation.h"

SpecularElement::SpecularElement(double kz, const Instrument& instrument, bool computable)
    : m_polarization(instrument.beam().getPolarization(),
                     instrument.detector().detectionProperties().analyzerOperator())
    , m_intensity(0.0)
    , m_computable(computable)
    , m_kz_computation(
          [kz](const SliceStack& slices) { return Compute::Kz::computeKzFromSLDs(slices, kz); })
{
}

SpecularElement::SpecularElement(double wavelength, double alpha, const Instrument& instrument,
                                 bool computable)
    : m_polarization(instrument.beam().getPolarization(),
                     instrument.detector().detectionProperties().analyzerOperator())
    , m_intensity(0.0)
    , m_computable(computable)
    , m_kz_computation([k = vecOfLambdaAlphaPhi(wavelength, alpha, 0.0)](const SliceStack& slices) {
        return Compute::Kz::computeKzFromRefIndices(slices, k);
    })
{
}

SpecularElement::SpecularElement(const SpecularElement& other)
    : m_polarization(other.m_polarization)
    , m_intensity(other.m_intensity)
    , m_computable(other.m_computable)
    , m_kz_computation(other.m_kz_computation)
{
}

SpecularElement::SpecularElement(SpecularElement&& other) noexcept
    : m_polarization(std::move(other.m_polarization))
    , m_intensity(other.m_intensity)
    , m_computable(other.m_computable)
    , m_kz_computation(std::move(other.m_kz_computation))
{
}

SpecularElement::~SpecularElement() = default;

std::vector<complex_t> SpecularElement::produceKz(const SliceStack& slices)
{
    return m_kz_computation(slices);
}
